package com.millicom.microservice.service.adapters.controlador.numero.temporal;


import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.constraints.NotNull;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;


@RequestMapping("")
public interface NumeroTemporalAdaptador {
    @Operation(summary = "Obtener la respuesta de SimCard", description = "POST method to retrieve service Individuals", tags = {"Individuals"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "successful operation", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "204", description = "GET means no record found     DELETE means sucessfully deletion"),
            @ApiResponse(responseCode = "400", description = "Bad Request", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "401", description = "Unauthorized", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "403", description = "Forbidden", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "404", description = "Not Found", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "405", description = "Method Not allowed", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "409", description = "Conflict", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "500", description = "Internal Server Error", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "503", description = "Service Unavailable", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class))),
            @ApiResponse(responseCode = "504", description = "Gateway Timeout", content = @Content(mediaType = "application/json;charset=utf-8", schema = @Schema(implementation = Error.class)))})
    /**
     * Obtiene información de la tarjeta SIM a partir del cuerpo de la solicitud.
     *
     * @param cuerpo El cuerpo de la solicitud que contiene información de la tarjeta SIM.
     * @return Una ResponseEntity con la información de la tarjeta SIM o una estructura de error en caso de problemas.
     */
    @PostMapping(value = "${service.numerotemporal.patch}**", consumes = {MediaType.TEXT_XML_VALUE}, produces = {"text/xml;charset=utf-8"})
    ResponseEntity<Object> consultarNumeroTemporal(@RequestBody @NotNull String cuerpo, HttpServletRequest solicitudHttp);


    @GetMapping(path = "${service.numerotemporal.patch}**", produces = {"text/xml;charset=utf-8"})
    ResponseEntity<Object> obtenerFirma(HttpServletRequest solicitudHttp);


}
