package com.millicom.microservice.service.adapters.soap.service.interceptor;

import com.readinessit.library.logging.application.services.LoggerService;
import com.readinessit.library.messaging.logs.MessagingLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.ws.server.endpoint.interceptor.PayloadTransformingInterceptor;
import org.springframework.xml.transform.ResourceSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import javax.xml.transform.Templates;
import javax.xml.transform.TransformerFactory;

import static com.millicom.microservice.service.external.services.common.Constants.MODULO;

@Component
public class CustomPayloadTransformingInterceptor extends PayloadTransformingInterceptor {


    private final MessagingLogger messagingLogger;

    private Resource requestXslt;
    private Resource responseXslt;
    private Templates requestTemplates;
    private Templates responseTemplates;

    @Autowired
    public CustomPayloadTransformingInterceptor(MessagingLogger messagingLogger) {
        this.messagingLogger = messagingLogger;
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        if (this.requestXslt == null && this.responseXslt == null) {
            // LoggerService.logBusiness(MODULO, "NULL", requestXslt);
        } else {
            TransformerFactory transformerFactory = this.getTransformerFactory();
            XMLReader xmlReader = XMLReaderFactory.createXMLReader();
            xmlReader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
            ResourceSource responseSource;
            if (this.requestXslt != null) {
                Assert.isTrue(this.requestXslt.exists(), "requestXslt \"" + this.requestXslt + "\" does not exit");
                LoggerService.logBusiness(MODULO, "Transforming request using " + this.requestXslt, null);


                responseSource = new ResourceSource(xmlReader, this.requestXslt);
                this.requestTemplates = transformerFactory.newTemplates(responseSource);

                LoggerService.logBusiness(MODULO, "Request", requestXslt);
            }

            if (this.responseXslt != null) {
                Assert.isTrue(this.responseXslt.exists(), "responseXslt \"" + this.responseXslt + "\" does not exit");
                LoggerService.logBusiness(MODULO, "Transforming response using " + this.responseXslt, null);
                responseSource = new ResourceSource(xmlReader, this.responseXslt);
                this.responseTemplates = transformerFactory.newTemplates(responseSource);
                LoggerService.logBusiness(MODULO, "Response", responseXslt);
            }


            //  this.messagingLogger.createMessageLog(requestBodyUriSpec, sentDate, request, adapterDefinition, url, clientResponse, respBody);
            // this.messagingLogger.createMessageLog(requestBodyUriSpec, sentDate, requestXslt, adapterDefinition, url, clientResponse, responseXslt);

        }
    }
}



