package com.millicom.microservice.service.controller.core;

import com.millicom.microservice.service.adapters.dto.common.ErrorStructure;
import com.millicom.microservice.service.service.common.GenerateErrorService;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.beans.ConversionNotSupportedException;
import org.springframework.beans.TypeMismatchException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.converter.HttpMessageNotWritableException;
import org.springframework.lang.Nullable;
import org.springframework.validation.BindException;
import org.springframework.web.HttpMediaTypeNotAcceptableException;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingPathVariableException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.context.request.async.AsyncRequestTimeoutException;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.multipart.support.MissingServletRequestPartException;
import org.springframework.web.servlet.NoHandlerFoundException;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;
//import com.readinessit.library.internationalization.interceptor.RequestInterceptor;


@RestControllerAdvice
public class BaseController {
    @Autowired
    GenerateErrorService generateErrorService;
    private ResponseEntityExceptionHandler responseEntityExceptionHandler;

   // @Autowired
  //  RequestInterceptor requestInterceptor;

    public BaseController() {
        responseEntityExceptionHandler = new ResponseEntityExceptionHandler() {
        	 
        	@Override
             protected ResponseEntity<Object> handleExceptionInternal(Exception ex, @Nullable Object body, HttpHeaders headers, HttpStatusCode status, WebRequest request) {
                 return handleErrorResponseCreation(ex, headers, status, request);
             }
        };
    }

    @ExceptionHandler({HttpMediaTypeNotSupportedException.class, HttpMediaTypeNotAcceptableException.class, MissingPathVariableException.class, MissingServletRequestParameterException.class, ServletRequestBindingException.class, ConversionNotSupportedException.class, TypeMismatchException.class, HttpMessageNotReadableException.class, HttpMessageNotWritableException.class, MethodArgumentNotValidException.class, MissingServletRequestPartException.class, BindException.class, AsyncRequestTimeoutException.class})
    public final ResponseEntity<Object> handleException(Exception ex, WebRequest request, HandlerMethod handlerMethod) throws Exception {
        ResponseEntity<Object> error = responseEntityExceptionHandler.handleException(ex, request);

        if(error != null ) {
            ErrorStructure body = error.hasBody() ? (ErrorStructure) error.getBody() : null;
            if(body != null)
         //       body.getLonginusInternalErrorCode().set_atMoreInfo(handlerMethod.getBeanType().getName() + ":" + handlerMethod.getMethod().getName());
            return error;
        }
        throw ex;
    }

    @ExceptionHandler({HttpRequestMethodNotSupportedException.class, NoHandlerFoundException.class})
    public final ResponseEntity<Object> handleException(Exception ex, WebRequest request) throws Exception {
      //  requestInterceptor.preHandle(((ServletWebRequest)request).getRequest(), ((ServletWebRequest)request).getResponse(), null);
        return responseEntityExceptionHandler.handleException(ex, request);
    }

    @ExceptionHandler({Exception.class})
    public final ResponseEntity<Object> handleUnknownException(Exception ex, WebRequest request, HandlerMethod handlerMethod) {
        ResponseEntity<Object> error = handleErrorResponseCreation(ex, null, HttpStatus.INTERNAL_SERVER_ERROR, request);

        ErrorStructure body = error.hasBody() ? (ErrorStructure) error.getBody() : null;
        if(body != null){}
         //  body.getLonginusInternalErrorCode().set_atMoreInfo(handlerMethod.getBeanType().getName() + ":" + handlerMethod.getMethod().getName());
        return error;
    }

    @ExceptionHandler({ HttpClientErrorException.class, HttpServerErrorException.class })
	public final ResponseEntity<Object> handleClientError(HttpClientErrorException ex, WebRequest request, HandlerMethod handlerMethod) {
		ResponseEntity<Object> error = handleErrorResponseCreation(ex, null, ex.getStatusCode(), request);

		ErrorStructure body = error.hasBody() ? (ErrorStructure) error.getBody() : null;
		if (body != null){}
			//body.getLonginusInternalErrorCode().set_atMoreInfo(handlerMethod.getBeanType().getName() + ":" + handlerMethod.getMethod().getName());
		
		//body.getBackendErrorCode().setErrorDescription(ex.getResponseBodyAsString());
		return error;
	}
    
    private ResponseEntity<Object> handleErrorResponseCreation(Exception ex, HttpHeaders headers, HttpStatusCode status, WebRequest request) {
        String stacktrace = null;
        if (HttpStatus.INTERNAL_SERVER_ERROR.equals(status)) {
            request.setAttribute("javax.servlet.error.exception", ex, 0);
            stacktrace = ExceptionUtils.getStackTrace(ex);
        }

        String simpleStackTrace = ex.getMessage();
        ErrorStructure error = generateErrorService.generateError(status, "MT_ERR_GEN_0001", new String[]{simpleStackTrace}, stacktrace, getClass());
        return new ResponseEntity<>(error, headers, status);
    }
}
