package com.millicom.microservice.service.controller.core;

import com.millicom.microservice.service.adapters.controlador.numero.temporal.NumeroTemporalRecurso;
import com.millicom.microservice.service.adapters.dto.common.ErrorStructure;
import com.millicom.microservice.service.service.common.GenerateErrorServiceImp;
import com.millicom.microservice.service.service.numero.propiedades.map.PropiedadesRespuesta;
import jakarta.servlet.RequestDispatcher;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.servlet.error.ErrorController;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import static com.millicom.microservice.service.external.services.common.Constants.ERROR_CODE_EXCEPTION;
import static com.millicom.microservice.service.external.services.common.Constants.ERROR_CODE_NOT_FOUND;

@RestController
public class ErrorPageController implements ErrorController {

    @Autowired
    PropiedadesRespuesta propiedadesRespuesta;

    @Autowired
    GenerateErrorServiceImp generateErrorService;

    @Autowired
    NumeroTemporalRecurso createOrderRecurso;

    @RequestMapping(value = "/error", method = {
            RequestMethod.GET,
            RequestMethod.POST,
            RequestMethod.DELETE,
            RequestMethod.PATCH,
            RequestMethod.OPTIONS})
    public ResponseEntity<Object> handleError(HttpServletRequest request) {

        Object status = request.getAttribute(RequestDispatcher.ERROR_STATUS_CODE);

        if (status != null) {

            int statusCode = Integer.parseInt(status.toString());
            HttpStatus httpStatus = HttpStatus.valueOf(statusCode);


            if (httpStatus == HttpStatus.NOT_FOUND) {
                // String requestedPath = request.getAttribute("javax.servlet.error.request_uri").toString();
                //ErrorStructure error = generateErrorService.generateError(httpStatus, ERROR_CODE_NOT_FOUND, null, "There is no controller for this operation" + requestedPath, getClass());
                 return ResponseEntity.status(httpStatus).body("The requested resource could not be found.");
              //  return ResponseEntity.status(200).contentType(MediaType.TEXT_XML).body(createOrderRecurso.obtenerFirma(request));
            }


            if (propiedadesRespuesta.getErrors().containsKey(String.valueOf(httpStatus.value()))) {
                ErrorStructure error = generateErrorService.generateError(httpStatus, ERROR_CODE_NOT_FOUND, null, "N/A", getClass());
                return ResponseEntity.status(httpStatus).body(error);
            }

        }

        ErrorStructure error = generateErrorService.generateError(HttpStatus.INTERNAL_SERVER_ERROR, ERROR_CODE_EXCEPTION, null, "Default /error mapping", getClass());
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(error);
    }
}
