package com.millicom.microservice.service.service.common;

import com.millicom.microservice.service.adapters.dto.common.ErrorStructure;
import com.millicom.microservice.service.adapters.dto.common.OperationException;
import com.millicom.microservice.service.service.numero.propiedades.Propiedades;
import com.readinessit.library.logging.application.services.LoggerService;
import io.micrometer.tracing.Tracer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.stereotype.Service;

import static com.millicom.microservice.service.external.services.common.Constants.MODULO;


@Service
public class GenerateErrorServiceImp implements GenerateErrorService {

    public final Tracer tracer;
    private final Propiedades propiedades;

    @Value("${modules.common.i18n.errorMessage.default.lang:en-US}")
    public String defaultCountry;

    @Autowired
    public GenerateErrorServiceImp(Tracer tracer, Propiedades propiedades) {
        this.tracer = tracer;
        this.propiedades = propiedades;
    }


    @Override
    public ErrorStructure generateError(HttpStatusCode httpStatus, String errorCode, String[] params, String description, Class ownerObject) {
        return generarErrorNuevo((HttpStatus) httpStatus, description);
    }


    /**
     * Genera una estructura de error para una respuesta HTTP y descripción proporcionadas.
     *
     * @param httpStatus  El estado HTTP de la respuesta.
     * @param description La descripción del error.
     * @return Una estructura de error que contiene el estado HTTP y la lista de errores correspondiente.
     */
    public ErrorStructure generarErrorNuevo(HttpStatus httpStatus, String description) {
        ErrorStructure errorStructure = new ErrorStructure();
        errorStructure.setHttpStatus(httpStatus);
        String status = String.valueOf(httpStatus.value());
        String error = propiedades.getMapResponse().getOrDefault(status, null);
        if (error != null) {
            errorStructure.setHttpStatus(httpStatus);
            errorStructure.setValor(error);
        }

        if (errorStructure.getValor() == null) {
            LoggerService.logBusiness(MODULO, "Devolviendo Respuesta por Default", null);
            errorStructure.setValor(propiedades.getMapResponse().getOrDefault("default", "SoapFault Exception"));
            errorStructure.setHttpStatus(HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return errorStructure;
    }

    /**
     * Obtiene la lista de errores a partir del estado y la respuesta proporcionados.
     *
     * @param status   El estado de la respuesta.
     * @param response La respuesta en formato JSON.
     * @return Una lista de errores, o una lista vacía si el estado no está mapeado o hay problemas de procesamiento.
     */


    /**
     * Método para generar y lanzar una excepción de operación.
     *
     * @param httpStatus Estado HTTP que se utilizará en la excepción.
     * @throws OperationException Excepción de operación generada con el error correspondiente.
     */
    @Override
    public void generarExcepcion(HttpStatus httpStatus, String descripcion) {
        throw new OperationException(httpStatus, generateError(
                httpStatus,
                descripcion,
                new String[]{descripcion},
                descripcion,
                this.getClass()
        ));
    }

}

