package com.millicom.microservice.service.service.messaging;

import com.readinessit.library.commoninterfaces.common.ISessionInfo;
import com.readinessit.library.commoninterfaces.messages.MessageIdentifiers;
import io.micrometer.tracing.Tracer;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.stereotype.Service;

import java.util.UUID;

@Service
public class MessageIdentifiersImp implements MessageIdentifiers {

    @Autowired
    ISessionInfo sessionDTO;

    @Autowired
    Tracer tracer;

    @Override
    public String calculateInternalId(HttpServletRequest httpServletRequest) {
        return handleInternalId();
    }

    @Override
    public String calculateInteractionId(HttpServletRequest httpServletRequest) {
        return  handleInteractionId();
    }

    @Override
    public String calculateNegotiationId(HttpServletRequest httpServletRequest) {
        return sessionDTO.getExternalId();
    }

    @Override
    public String calculateUserId(ServerHttpResponse httpServletRequest) {
        return sessionDTO.getUserId();
    }

    @Override
    public String calculateInternalId(ServerHttpResponse httpServletResponse) {
        return handleInternalId();
    }

    @Override
    public String calculateInteractionId(ServerHttpResponse httpServletRequest) {
        return  handleInteractionId();
    }

    @Override
    public String calculateNegotiationId(ServerHttpResponse httpServletRequest) {
        return null;
    }

    @Override
    public String calculateUserId(HttpServletRequest httpServletResponse) {
        String userId = httpServletResponse.getHeader("x-consumer-username");
        if(sessionDTO.getUserId() == null)
            sessionDTO.setUserId(userId);

        return userId;
    }

    private String handleInteractionId() {
        if(sessionDTO.getTransactionId() == null)
            sessionDTO.setTransactionId(UUID.randomUUID().toString());

        return sessionDTO.getTransactionId();
    }

    private String handleInternalId() {
        var currentSpan = tracer.currentSpan();
        if (currentSpan != null && sessionDTO.getInternalId() == null)
            sessionDTO.setInternalId(currentSpan.context().traceId());

        return sessionDTO.getInternalId();
    }
}