package com.millicom.microservice.service.helper;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.github.tomakehurst.wiremock.WireMockServer;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StreamUtils;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Map;

public class BaseTestAPI {

    @Value("${configs.MOCK_PORT}")
    public int DEFAULT_PORT;
    protected WireMockServer externalSystemsMock;

    @Autowired
    protected ObjectMapper objectMapper;


    public ObjectMapper getObjectMapper() {
        return objectMapper;
    }

    @BeforeEach
    protected void preSetup() throws Exception {
        //Initialize Mock
        if (externalSystemsMock == null) {
            externalSystemsMock = new WireMockServer(DEFAULT_PORT);
        }

        externalSystemsMock.start();
        JavaTimeModule module = new JavaTimeModule();
        objectMapper.registerModule(module);

    }

    @AfterEach
    protected void afterTest() {
        externalSystemsMock.stop();
    }


    public String resourceToString(org.springframework.core.io.Resource resource) throws IOException {
        return StreamUtils.copyToString(resource.getInputStream(), StandardCharsets.UTF_8);
    }

    public String replaceDynamicParams(String endpoint, Map<String, String> dynamicParams) {
        UriComponentsBuilder builder = UriComponentsBuilder.fromUriString("");
        builder.path(endpoint);
        if (dynamicParams != null && dynamicParams.size() > 0) {
            endpoint = builder.build(false).expand(dynamicParams).toUriString();
        }
        return endpoint;
    }
}
